﻿using DTOs;
using Microsoft.AspNetCore.Mvc;
using OrderCriterias;

namespace WebApi.Controllers
{
    public class ChapterController : ControllerBase
    {
        private Unit unity;

        private readonly Logger<ChapterController> logger;

        public ChapterController(Unit unit, Logger<ChapterController> logger)
        {
            this.unity = unit;
            this.logger = logger;
        }

        /// <summary>
        /// add a chapter
        /// </summary>
        /// <param name="chapter">the chapter to add</param>
        /// <returns>
        /// status code :
        /// 200 if the chapter is added
        /// 202 if the chapter is added
        /// 208 if the chapter was already in the database
        /// 500 if there was an internal error that doesn't allow to continue
        /// return content :
        /// no content when status code = 200
        /// chapter added when status code = 202
        /// chapter already in the database that equals the one we wanted to add when status code = 208
        /// no content when status code = 500
        /// </returns>
        public async Task<IActionResult> PostChapter(ChapterDto chapter)
        {
            int count = unity.getNbChapters(); // count : number of elements before opperation
            var tmp = await unity.addChapter(chapter); //  tmp : chapter recieve by the addChapter method
            if (unity.getNbChapters() == count) // <=> not added
            {
                if (tmp.Name == chapter.Name)
                {
                    // it was already in the database
                    logger.LogInformation(message: $"want to add a chapter already in the database : {tmp.ToString()}");
                    return StatusCode(208, tmp);
                }
                else
                {
                    // we recieve a chapter already in the database
                    // that should be equal to the chapter that we
                    // wanted to add but it isn't
                    logger.LogCritical(message: "controller's add fail (already in the database) but\nchapter" +
                        $" recieve isn't the same as the one we wanted to add.\n" +
                        $"Chapter recieve : {tmp.ToString()}\nchapter to add : {chapter.ToString()}");
                    return StatusCode(500);
                }
            }
            // added
            if (tmp.Name == chapter.Name)
            {
                logger.LogTrace(message: $"chapter added : {tmp.ToString()}");
                // the chapter has been added and we recieved him
                return StatusCode(202, tmp);
            }
            else
            {
                // the chapter may be added but we do not recieved him
                if (unity.getChapter(chapter.Name) != null)
                {
                    // he is added
                    logger.LogError(message: $"chapter added but not recieved");
                    return Ok(); // not 202 to make a difference between 2 cases
                }
                else
                {
                    // he is not added
                    logger.LogCritical(message: "chapter that we wanted to add not added\nand we have added another one");
                    if (unity.getChapter(chapter.Name) == null) // <=> not added
                        return StatusCode(500);
                }
            }
            logger.LogError(message: "this case should not append");
            return StatusCode(500);
        }

        /// <summary>
        /// get a part of all chapters
        /// </summary>
        /// <param name="page">the actual page</param>
        /// <param name="count">number of T element in a page</param>
        /// <param name="orderCriteria">the order criteria</param>
        /// <returns>
        /// status code :
        /// 200 if we got a set
        /// 204 if we got an empty set
        /// 400 if we got nothing
        /// 
        /// return content :
        /// a tuple of the number of page and
        /// all chapters in the database for
        /// the page nb if status code = 200
        /// the number of page for count element if status code = 204
        /// nothing if status code = 400
        /// </returns>
        public async Task<IActionResult> GetSomeChapters(int page, int count = 10, ChapterOrderCriteria orderCriteria = ChapterOrderCriteria.ById)
        {
            var tmp = await unity.getChapters(page, count, orderCriteria);
            if (tmp.chapters == null)
            {
                logger.LogInformation(message: "get chapter : bad request (page or/and count incorrect)");
                return BadRequest(tmp.nbPages);
            }
            else if (tmp.chapters.Count() == 0)
            {
                logger.LogWarning(message: $"get chapter : no content. number of element : {unity.getNbChapters()}, page wanted : {page}, number of elements in a page : {count}");
                return NoContent();
            }
            else
            {
                logger.LogTrace(message: $"get chapters : page = {page}, count = {count}, order criteria = {orderCriteria switch
                {
                    ChapterOrderCriteria.ById => "byId",
                    ChapterOrderCriteria.ByName => "byName",
                    _ => "none"
                }}");
                return Ok(tmp);
            }
        }

        /// <summary>
        /// delete a chapter
        /// </summary>
        /// <param name="id">the id of the chapter to delete</param>
        /// <returns>
        /// <returns>
        /// status code :
        /// 200 if the chapter is removed
        /// 204 if the chapter is removed
        /// 400 if the id is incorrect
        /// 500 if there was an internal error that doesn't allow to continue
        /// return content :
        /// aministrator deleted when status code = 200
        /// no content when status code = 204
        /// no content when status code = 400
        /// no content when status code = 500
        /// </returns>
        public async Task<IActionResult> DeleteChapter(int id)
        {
            if (id < 0)
            {
                logger.LogError("want to delete a chapter with an id less than 0");
                return BadRequest();
            }
            int count = unity.getNbChapters(); // count : number of elements before opperation
            var tmp = await unity.removeChapter(id);
            if (tmp == null) // we don't recieve the chapter
            {
                if (unity.getNbChapters() == count) // he is not deleted 
                {
                    if (unity.getChapter(id) != null)
                    {
                        logger.LogCritical(message: "remove chapter fail : chapter not removed and not recieved");
                        return StatusCode(500);
                    }
                    else
                    {
                        logger.LogInformation(message: "trying to remove a chapter with an id who don't exist");
                        return BadRequest();
                    }
                }
                else // he may be deleted
                {
                    if (unity.getChapter(id) == null) // he must be deleted
                    {
                        logger.LogError(message: "chapter removed but not returned");
                        return NoContent();
                    }
                    else // he is not deleted
                    {
                        logger.LogCritical(message: "remove chapter fail : chapter to remove not remove\ninstead, anotherone is deleted and we recieved nothing");
                        return StatusCode(500);
                    }
                }
            }
            if (unity.getNbChapters() == count)
            {
                // <=> we have recieved a chapter which should be deleted
                // but since we have the same number of chapter than
                // before deletion, it isn't deleted
                logger.LogCritical(message: $"chapter \"{tmp.ToString()}\"should be delete but it isn't");
                return StatusCode(500);
            }
            logger.LogTrace(message: $"chapter removed {tmp.ToString()}");
            return Ok(tmp);
        }

        /// <summary>
        /// update a chapter
        /// </summary>
        /// <param name="id">id of the chapter to update</param>
        /// <param name="chapter">a chapter who contains all properties to change</param>
        /// <returns>
        /// status code :
        /// 200 if the chapter is modified
        /// 204 if we wanted to modify a chapter who don't exist
        /// 208 if the new username of the chapter is already used
        /// 500 if there was an internal error that doesn't allow to continue
        /// 
        /// return content :
        /// the chapter modified when status code = 200
        /// nothing when status code = 204
        /// the chapter that already have the username when status code = 208
        /// nothing when status code = 500
        /// </returns>
        public async Task<IActionResult> PutChapter(int id, ChapterDto chapter)
        {
            var tmp = await unity.getChapter(chapter.Name);
            if (tmp != null)
            {
                logger.LogTrace(message: "Want to modify into a chapter who already exist");
                return StatusCode(StatusCodes.Status208AlreadyReported, tmp);
            }
            tmp = await unity.updateChapter(id, chapter.Name);
            if (tmp == null)
            {
                tmp = await unity.getChapter(id);
                if (tmp == null) // Ok, it don't exist
                {
                    logger.LogWarning(message: "Want to modify a chapter who don't exist");
                    return NoContent();
                }
                else
                {
                    if (tmp.Name == chapter.Name)
                    {
                        logger.LogError(message: "Chapter changed but not recieved");
                        return Ok(tmp);
                    }
                    else
                    {
                        logger.LogCritical(message: "chapter haven't changed and we recieved nothing");
                        return StatusCode(500);
                    }
                }
            }
            else if (tmp.Id == id) // we recieve him
            {
                if (tmp.Name == chapter.Name) // he is changed
                {
                    logger.LogTrace(message: $"chapter with id {id} modified in {chapter.ToString()}");
                    return Ok(tmp);
                }
                else // he haven't changed
                {
                    var tmp2 = (await unity.getChapter(chapter.Name));
                    if (tmp2 != null) // it change another chapter
                        logger.LogCritical(message: "chapter should have changed but he haven't. Instead, another one changed");
                    else // nothing have changed
                        logger.LogCritical(message: "chapter should have changed but he haven't and we recieved him");
                    return StatusCode(500);
                }
            }
            else // we recieve another one
            {
                var tmp2 = await unity.getChapter(id);
                if (tmp2 == null)
                { // ok, he d'ont exist.
                    logger.LogError(message: "Want to modify a chapter who don't exist but recieved a random one");
                    return NoContent();
                }
                else if (tmp2.Name == chapter.Name)
                {
                    logger.LogError(message: $"chapter modified but recieved a random one");
                    return Ok(tmp2);
                }
                else
                {
                    logger.LogCritical(message: "chapter that we wanted to modify not modified");
                    return StatusCode(500);
                }
            }
        }

        /// <summary>
        /// get a chapter
        /// </summary>
        /// <param name="name">the name of the chapter</param>
        /// <returns>
        /// status code :
        /// 200 when we got a chapter
        /// 204 when we got null
        /// 
        /// return content :
        /// the chapter that correspond to the name when status code = 200
        /// nothing when status code 204
        /// </returns>
        public async Task<IActionResult> GetChapter(string name)
        {
            var tmp = await unity.getChapter(name);
            if(tmp == null) return NoContent();
            return Ok(tmp);
        }
    }
}
